// Libraries

//All 4 radios run on 9600 baud

#include <XBee.h>
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_LSM303_U.h>
#include <Adafruit_L3GD20_U.h>
#include <Adafruit_9DOF.h>

/* Assign a unique ID to the sensors */
Adafruit_9DOF                dof   = Adafruit_9DOF();
Adafruit_LSM303_Accel_Unified accel = Adafruit_LSM303_Accel_Unified(30301);
Adafruit_LSM303_Mag_Unified   mag   = Adafruit_LSM303_Mag_Unified(30302);

/* Update this with the correct SLP for accurate altitude measurements */
float seaLevelPressure = SENSORS_PRESSURE_SEALEVELHPA;

// Xbee Initialization
#include "functions.h"

#define PKT_MAX_LEN 100

// ship select pin for SD logging
const int chipSelect = 4;

// object for holding packets received over xbee
Rx16Response reponse16 = Rx16Response();

// set XBee address of logger
uint16_t Logger_MY_Addr = 0x0001;

// set XBee address of this transmitter
uint16_t XBee_MY_Addr = 0x0002;

// PAN ID of xbee (must match all xbees)
uint16_t XBee_PAN_ID = 0x0B0B;

// initialize array to hold data received from xbee
uint8_t data[PKT_MAX_LEN];

// initalize an array to hold the data to send
uint8_t msg_payload[PKT_MAX_LEN];

uint8_t rcvd_data[PKT_MAX_LEN];

uint32_t last_transmit = 0;

int trans_interval = 1000;

int pin5 = 0;

// Sensor Initialization
void initSensors()
{
  if (!accel.begin())
  {
    /* There was a problem detecting the LSM303 ... check your connections */
    Serial.println(F("Ooops, no LSM303 detected ... Check your wiring!"));
    Serial2.println(F("Ooops, no LSM303 detected ... Check your wiring!"));
    while (1);
  }
  if (!mag.begin())
  {
    /* There was a problem detecting the LSM303 ... check your connections */
    Serial.println(F("Ooops, no LSM303 detected ... Check your wiring!"));
    Serial2.println(F("Ooops, no LSM303 detected ... Check your wiring!"));
    while (1);
  }
}

void setup() {
  Serial.begin(9600);
  Serial1.begin(9600);
  Serial2.begin(9600);

  initSensors();

  /*
  InitXBee is defined in the other file and will do the following things to the
  attached xbee:
  Set it's MY address to the value given in the second argument of InitXBee
  Set its PAN ID to 1111 (arbitrarily chosen, but all Xbees much match).

  Note that these setting are not saved to the non-volative memory on the xbee, so
  they will be discarded when the xbee powers down.

  You do not need to use this function if you attach an xbee which has already been
  properly configured, but this allows you to use any xbee without worrying/keeping
  track of if its been configured.
  */
  int XbeeStatus = InitXBee(XBee_MY_Addr, XBee_PAN_ID);
  if (!XbeeStatus) {
    Serial.println(F("Xbee initialized!"));
    Serial2.println(F("Xbee initialized!"));
  }
  else {
    /*
    If the Xbee inititalization completes sucessfully then it will return 0.
    Other values inidicate a problem with the initialization, which will be
    printed, but otherwise the program will continue. For a properly configured
    xbee, it may not be an issue if the intitialization doesn't complete (though
    the usual cause for it not completing is that the jumper/switch on the xbee
    shield is in the wrong position, which would mean it wouldn't regaurdless).
    */
    Serial.print(F("Xbee failed to initialize with Error code: "));
    Serial.println(XbeeStatus);
    Serial2.print(F("Xbee failed to initialize with Error code: "));
    Serial2.println(XbeeStatus);
  }
}

void loop() {
  sensors_event_t accel_event;
  sensors_event_t mag_event;
  sensors_vec_t   orientation;

  /* Calculate pitch and roll from the raw accelerometer data */
  accel.getEvent(&accel_event);
  float roll, pitch, heading;

  if (dof.accelGetOrientation(&accel_event, &orientation))
  {
    /* 'orientation' should have valid .roll and .pitch fields */
    roll = orientation.roll;
    pitch = orientation.pitch;
  }

  /* Calculate the heading using the magnetometer */
  mag.getEvent(&mag_event);
  if (dof.magGetOrientation(SENSOR_AXIS_Z, &mag_event, &orientation))
  {
    /* 'orientation' should have valid .heading data now */
    heading = orientation.heading;
  }

  // Create String to store float values for Roll, Pitch, Heading angles.

  String IMU_String[] = { String(roll), String(pitch), String(heading) };
  //Loop to print data, loops every second.

  uint8_t offset = 0;
  pin5++;

  offset = addToTlm(roll, msg_payload, offset);
  offset = addToTlm(pitch, msg_payload, offset);
  offset = addToTlm(heading, msg_payload, offset);

  // read messages
  if (readMsg(rcvd_data, 1) > 0) {
  
      // its a request for a HK packet
      if (rcvd_data[4] == 0xAB && rcvd_data[5] == 0xCD) {
        Serial.print(F("Sending HK Pkt to "));
        Serial.println(extract_uint16(rcvd_data, 2));
        Serial2.print(F("Sending HK Pkt to "));
        Serial2.println(extract_uint16(rcvd_data, 2));
        sentHKpkt(extract_uint16(rcvd_data, 2));
      }
      // otherwise, log the data we received
      else {
        String data_Str = data2string(rcvd_data);
        
        // write the data to the radio 1
        Serial.println(data_Str);
        // write data to radio 2
        Serial2.println(data_Str);
      }
    }
  //if(last_transmit > millis()){
  // last_transmit = 0; 
  //}

  if(millis() - last_transmit > trans_interval){
      last_transmit = millis();
    // send data to logger
    sendData(Logger_MY_Addr, msg_payload, offset * sizeof(uint8_t));
    
    // send data via radio 1
    Serial.println("------");
    
    Serial.println("Radio 1");
  
    for (int count = 0; count < 3; count++) {
        char* labels[] = { "Roll: ", "Pitch: ", "Heading: " };
        Serial.println(labels[count]);
        Serial.print(IMU_String[count]);
        Serial.print(" degrees");
        Serial.println();
      }
      
    Serial.println("======");
  
    Serial2.println("------");
    
    Serial2.println("Radio 2");
    
    // send data via radio 2
    for (int counter = 0; counter < 3; counter++) {
      char* labelz[] = { "Roll: ", "Pitch: ", "Heading: " };
      Serial2.println(labelz[counter]);
      Serial2.print(IMU_String[counter]);
      Serial2.print(" degrees");
      Serial2.println();
    }
    
    Serial2.println("======");
  }
  delay(10);
}

